#!/usr/bin/env python3

# system modules

# internal modules

# external modules


### Temperature ###
zero_celsius_in_kelvin = 273.15
r"""
Zero degrees Celsius in Kelvin

:math:`\left[T_\mathrm{0^\circ C}\right] = K`

https://de.wikipedia.org/wiki/Kelvin
"""

### Gas constants ###
gas_constant_univeral = 8.3144598
r"""
universal gas constant 

:math:`\left[R^*\right] = \frac{J}{mol\,K}`

https://de.wikipedia.org/wiki/Gaskonstante
"""

gas_constant_water_vapour = 461.4
r"""
specific gas constant of water vapour

:math:`\left[R_\mathrm{watervapour}\right] = \frac{J}{kg\,K}`

https://de.wikipedia.org/wiki/Gaskonstante
"""

gas_constant_dry_air = 287.058
r"""
specific gas constant of dry air

:math:`\left[R_\mathrm{dryair}\right] = \frac{J}{kg\,K}`

https://de.wikipedia.org/wiki/Gaskonstante
"""

gas_constant_co2 = 188.9
r"""
specific gas constant of carbon dioxide

:math:`\left[R_\mathrm{CO2}\right] = \frac{J}{kg\,K}`

https://de.wikipedia.org/wiki/Gaskonstante
"""

### radiation ###
stefan_boltzmann_constant = 5.670367e-8
r"""
Stefan-Boltzmann constant for blackbody-radiation

:math:`\left[\sigma\right] = \frac{W}{m^2\,K^4}`

https://de.wikipedia.org/wiki/Stefan-Boltzmann-Gesetz
"""

solar_constant_toa = 1367
r""" 
solar constant at top of atmosphere

:math:`\left[I_0\right] = \frac{W}{m^2}`

https://de.wikipedia.org/wiki/Solarkonstante
"""

### material ###
density_water_0C = 999.87
r"""
density of water at :math:`0^\circ C` and standard sea-level atmospheric
pressure

:math:`\left[\rho_\mathrm{water,0^\circ C}\right] = \frac{kg}{m^3}`

https://water.usgs.gov/edu/density.html
"""

density_water_4C = 1000.0
r"""
density of water at :math:`4^\circ C` and standard sea-level atmospheric
pressure

:math:`\left[\rho_\mathrm{water,4^\circ C}\right] = \frac{kg}{m^3}`

https://water.usgs.gov/edu/density.html
"""



#!/usr/bin/env python3

# system modules
import warnings

# internal modules
from .constants import zero_celsius_in_kelvin

# external modules
import numpy as np

def cel2kel(T):
    """ 
    Convert temperatures in degrees Celsius to degrees Kelvin

    Args:
        T (numpy.ndarray): temperature in degrees Celsius

    Returns:
        numpy.ndarray : given temperature converted to degrees Kelvin
    """
    # convert to an array
    celsius = np.asanyarray(T).astype(float)
    oldshape = celsius.shape
    celsius = celsius.reshape(-1)
    # convert to kelvin
    kelvin = celsius + zero_celsius_in_kelvin
    # replace invalid temperatures
    invalid = kelvin < 0
    if invalid.any():
        warnings.warn("{} temperatures below 0 Kelvin masked with NaN".format(
            invalid.sum()))
        kelvin[invalid] = np.nan
    return kelvin.reshape(oldshape)

def kel2cel(T):
    """ 
    Convert temperatures in degrees Kelvin to degrees Celsius

    Args:
        T (numpy.ndarray): temperature in degrees Kelvin

    Returns:
        numpy.ndarray : given temperature converted to degrees Celsius
    """
    # convert to an array
    kelvin = np.asanyarray(T).astype(float)
    oldshape = kelvin.shape
    kelvin = kelvin.reshape(-1)
    invalid = kelvin < 0
    if invalid.any():
        warnings.warn("{} temperatures below 0 Kelvin masked with NaN".format(
            invalid.sum()))
        kelvin[invalid] = np.nan
    # convert to kelvin
    celsius = kelvin - zero_celsius_in_kelvin
    # replace invalid temperatures
    return celsius.reshape(oldshape)



#!/usr/bin/env python3

# system modules
import warnings

# internal modules
from .temperature import kel2cel

# external modules
import numpy as np

def saturation_water_vapour_pressure(T, over = "auto", param = "magnus"):
    r""" 
    Saturation water vapour pressure over ``over`` according to the
    parametisation ``param``.
    

    Args:
        T (numpy.ndarray): temperature in degrees Kelvin
        over (str, optional): use the parameterisation over ``"water"`` or
            ``"ice"`` or ``"auto"`` (the default) to use ``"water"`` for
            temperatures over and including :math:`0^\circ C` and ``"ice"``
            below.
        param (str, optional): the parameterisation to use. Defaults to
            ``"magnus"``. Available parameterisations are: 
            
            ``"magnus"`` 
                `Magnus-Formula`_


    Returns:
        numpy.ndarray : the saturation water vapour pressure 
        :math:`\left[e_\mathrm{s}\right] = Pa`

    .. _Magnus-Formula: https://de.wikipedia.org/w/index.php?oldid=162431872
    """
    celsius = kel2cel(T) # convert to Celsius

    if param == "magnus":
        # the Magnus parameterisations
        params = { 
            "water": {
                "A": 6.112e2, # Pa
                "B": 17.62, # C^-1
                "C": 243.12, # C
                "low": -45, # lower bound C
                "high": 65, # upper bound C
                },
            "ice": {
                "A": 6.112e2, # Pa
                "B": 22.46, # C^-1
                "C": 272.62, # C
                "low": -65, # lower bound C
                "high": 0.01, # upper bound C
                },
            }
        if over == "auto":
            over0celsius = celsius >= 0
            autoparam = {
                c:np.where(over0celsius,params["water"][c],params["ice"][c]) \
                    for c in params["water"].keys()}
            autoparam.update({
                "low":params["ice"]["low"],
                "high":params["water"]["high"],
                })
            params.update({"auto":autoparam})

        # get the appropriate parameterisation
        const = params.get(over)

        # check bounds
        outside = np.logical_or(celsius<const["low"],celsius>const["high"])
        if outside.any():
            warnings.warn("There are {} temperatures outside the " 
                "parameterisation scope [{:.2f};{:.2f}] °C".format(
                outside.sum(),const["low"],const["high"]))

        if not const: # pragma: no cover
            raise ValueError("unknown 'over' string '{}'".format(over))
            
        e_s = const["A"] * np.exp((const["B"]*celsius) / (const["C"] + celsius))
    else: # pragma: no cover
        raise ValueError("unknown parameterisation '{}'".format(param))

    return e_s
